@@ -1,3 +1,4 @@
+mod de;
 mod endian;
 mod helpers;
 mod indexed_map;
@@ -10,8 +11,6 @@ mod map;
 mod path;
 mod prefix;
 mod snapshot;
-mod snapshot_item;
-mod snapshot_map;
 
 pub use endian::Endian;
 #[cfg(feature = "iterator")]
@@ -29,7 +28,5 @@ pub use map::Map;
 pub use path::Path;
 #[cfg(feature = "iterator")]
 pub use prefix::{range_with_prefix, Bound, Prefix};
-pub use snapshot::Strategy;
-pub use snapshot_item::SnapshotItem;
 #[cfg(feature = "iterator")]
-pub use snapshot_map::SnapshotMap;
+pub use snapshot::{SnapshotItem, SnapshotMap, Strategy};
